/*
 * Name:     CActor.h
 *
 *
 * Class:    CActor
 * Impl:     CActor.cpp
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#pragma once
#ifndef CACTOR_H_
#define CACTOR_H_

#include "Channel/CAnimChannelPos.h"
#include "Channel/CAnimChannelEnable.h"
#include "Drawable/CDrawable.h"
#include "CIterator.h"
#include <wx/xml/xml.h>
#include <list>

class CDocument;

/*
 * Class:    CActor
 * 
 */
class CActor
{
public:
	CActor(const std::wstring &name);
	virtual ~CActor();

	void AddDrawable(CDrawable *drawable);
	void SetHDrawable(CDrawable *source);
	void Draw(wxDC *dc);
	CDrawable *HitTest(wxPoint pos);
	int XmlSaveKeyframeNum() {return mChannelP.GetNumKeyframes();}
	void XmlSave(wxXmlNode* keyframe, int here, bool first);

	// Set & Get for member variables
	void SetName(const std::wstring &name) {mName = name;}
	std::wstring GetName() const {return mName;}
	void SetPosition(wxPoint p) {mPosition = p;}
	wxPoint GetPosition() const {return mPosition;}
	void SetEnabled(bool e) {mEnabled = e;}
	bool GetEnabled() const {return mEnabled;}
	void TogEnabled() {mEnabled == false ? mEnabled = true : mEnabled = false;}
	void SetClickable(bool b) {mClickable = b;}
	bool GetClickable() const {return mClickable;}
	void SetDocument(CDocument *doc);
	CDocument *GetDocument() const {return mDocument;}
    void SetRoot(CDrawable *root) {mRoot = root;mDefaultRot = root->GetRotation();}
    CDrawable *GetRoot() {return mRoot;}
    CDrawable *GetText();
	void SetKeyframe();
	void GetKeyframe();
	void ClearKeyframe();
	void ClearChannels();
	void SetDefault();

    // Special function used to display children of the actor (see CViewActors::OnPaint)
    int DisplayDrawables(wxPaintDC &dc, int &y, int &textHeight);

private:
	CActor();

	std::wstring mName;
	wxPoint mPosition;
	wxPoint mDefaultPos;
	double mDefaultRot;
	bool mEnabled;
	bool mClickable;
	CDocument *mDocument;
	CDrawable *mRoot;

	std::list<CDrawable *> mDrawablesInOrder;
	std::list<CDrawable *> mDrawablesInHeirarchy;
    CAnimChannelPos mChannelP;
    CAnimChannelEnable mChannelE;

public:
    // An iterator for the drawables
	friend class Iterator;
	    class Iterator : public CIterator<CDrawable *>
	    {
	    public:
	        Iterator(CActor *actor) : mActor(actor)
	        {
	        	mIter = mActor->mDrawablesInOrder.begin();
	        }
	        virtual void First() {mIter = mActor->mDrawablesInOrder.begin();}
	        virtual void Next() {mIter++;}
	        virtual bool IsDone() {return mIter == mActor->mDrawablesInOrder.end();}
	        virtual CDrawable *Current() {return *mIter;}

	    private:
	        CActor* mActor;
	        std::list<CDrawable *>::iterator mIter;
	    };

	Iterator *GetDrawables() {return new Iterator(this);}

	friend class Iterator;
	    class Iterator2 : public CIterator<CDrawable *>
	    {
	    public:
	        Iterator2(CActor *actor) : mActor(actor)
	        {
	        	mIter = mActor->mDrawablesInHeirarchy.begin();
	        }
	        virtual void First() {mIter = mActor->mDrawablesInHeirarchy.begin();}
	        virtual void Next() {mIter++;}
	        virtual bool IsDone() {return mIter == mActor->mDrawablesInHeirarchy.end();}
	        virtual CDrawable *Current() {return *mIter;}

	    private:
	        CActor* mActor;
	        std::list<CDrawable *>::iterator mIter;
	    };

	Iterator2 *GetHDrawables() {return new Iterator2(this);}
};

#endif /* CACTOR_H_ */
