/*
 * Name:     CDocument.cpp
 *
 *
 * Class:    CDocument
 * Header:   CDocument.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CDocument.h"

using namespace std;

/*
 * Name :        CDocument::CDocument()
 * Description : Document class constructor.
 */
CDocument::CDocument()
{
    mSize = wxSize(800, 600);
}

/*
 * Name :        CDocument::CDocument()
 * Description : Document class destructor.
 */
CDocument::~CDocument()
{
	// Clean up, clean up, oh so fun
	for(list<CActor *>::iterator a=mActors.begin(); a!=mActors.end(); a++)
	{
		delete *a;
	}

	mActors.clear();
}

/*
 * Name :        CDocument::AddObserver()
 * Description : Add an observer to this document.
 * Parameters :  observer - The observer to add
 * Returns :     none
 */
void CDocument::AddObserver(CDocumentObserver *observer)
{
    mObservers.push_back(observer);
}

/*
 * Name :        CDocument::RemoveObserver()
 * Description : Remove an observer from this document.
 * Parameters :  observer - The observer to remove.
 * Returns :     none
 */
void CDocument::RemoveObserver(CDocumentObserver *observer)
{
    mObservers.remove(observer);
}

/*
 * Name :        CDocument::UpdateObservers()
 * Description : Update all observers to indicate the document has changed.
 * Parameters :  none
 * Returns :     none
 */
void CDocument::UpdateObservers()
{
    for(list<CDocumentObserver *>::iterator o=mObservers.begin(); o!=mObservers.end(); o++)
    {
        (*o)->UpdateObserver();
    }
}

/*
 * Name :        CDocument::AddActor()
 * Description : Adds an actor to the database
 * Parameters :  A pointer to the actor
 * Returns :     none
 */
void CDocument::AddActor(CActor *actor)
{
	mActors.push_back(actor);
	actor->SetDocument(this);
}
/*
 * Name :        CDocument::Draw()
 * Description : Draw a rectangle for testing
 * Parameters :  wxDC - a device context
 * Returns :     none
 */
void CDocument::Draw(wxDC *dc)
{
	for(list<CActor *>::iterator a=mActors.begin(); a!=mActors.end(); a++)
	{
		CActor *actor = *a;
		actor->Draw(dc);
	}
}

/*
 * Name :        CDocument::SetAnimationTime()
 * Description : Set the current animation time, forcing the animation of all
 *               objects to the current animation value.
 * Parameters :  time - The new time.
 * Returns :     none
 */
void CDocument::SetAnimationTime(double time)
{
    mTimeline.SetCurrentTime(time);
    UpdateObservers();

    // Update actors to this time.
    for(list<CActor *>::iterator a=mActors.begin(); a!=mActors.end(); a++)
    {
        CActor *actor = *a;
        actor->GetKeyframe();
    }
}
