/*
 * Name:     CDocument.h
 *
 *
 * Class:    CDocument
 * Impl:     CDocument.cpp
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#pragma once
#ifndef CDOCUMENT_H_
#define CDOCUMENT_H_

#include "CDocumentObserver.h"
#include "CActor.h"
#include "CIterator.h"
#include "CTimeline.h"
#include <list>

/*
 * Class:    CDocument
 * 
 */
class CDocument
{
public:
	CDocument();
	virtual ~CDocument();

	void AddObserver(CDocumentObserver *observer);
	void RemoveObserver(CDocumentObserver *observer);
	void UpdateObservers();
	void AddActor(CActor *actor);
	void Draw(wxDC *dc);

	void SetSize(wxSize s) {mSize = s;}
	wxSize GetSize() const {return mSize;}
	CTimeline *GetTimeline() {return &mTimeline;}
	void SetAnimationTime(double time);

private:
	wxSize mSize;
    std::list <CDocumentObserver *> mObservers;
    std::list<CActor *> mActors;
    CTimeline mTimeline;

public:
    // An iterator for the actors
	friend class Iterator;
	    class Iterator : public CIterator<CActor *>
	    {
	    public:
	        Iterator(CDocument *document) : mDocument(document)
	        {
	        	mIter = mDocument->mActors.begin();
	        }
	        virtual void First() {mIter = mDocument->mActors.begin();}
	        virtual void Next() {mIter++;}
	        virtual bool IsDone() {return mIter == mDocument->mActors.end();}
	        virtual CActor *Current() {return *mIter;}

	    private:
	        CDocument* mDocument;
	        std::list<CActor *>::iterator mIter;
	    };

	Iterator *GetActors() {return new Iterator(this);}
};

#endif /* CDOCUMENT_H_ */
