/*
 * Name:     CAnimChannelEnable.cpp
 *
 *
 * Class:    CAnimChannelEnable
 * Header:   CAnimChannelEnable.h
 * Created : Mar 19, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CAnimChannelEnable.h"
#include "../CTimeline.h"

/*
 * Name :        CAnimChannelEnable::CAnimChannelEnable()
 * Description : Animation channel enable constructor
 */
CAnimChannelEnable::CAnimChannelEnable()
{
}

/*
 * Name :        CAnimChannelEnable::~CAnimChannelEnable()
 * Description : Animation channel enable destructor
 */
CAnimChannelEnable::~CAnimChannelEnable()
{
}

/*
 * Name :        CAnimChannelEnable::GetNumKeyframes()
 * Description : Obtain the number of keyframes. This is the size of the keyframe list.
 * Parameters :  none
 * Returns :     Number of keyframes.
 */
int CAnimChannelEnable::GetNumKeyframes()
{
    return (int)mKeyframes.size();
}

/*
 * Name :        CAnimChannelEnable::GetKeyframe()
 * Description : Get a keyframe by index from the list of keyframes.
 * Parameters :  frame - The frame index
 * Returns :     Pointer to keyframe at that index.
 */
CAnimChannel::Keyframe *CAnimChannelEnable::GetKeyframe(int frame)
{
    return &mKeyframes[frame];
}

/*
 * Name :        CAnimChannelEnable::SetKeyframe()
 * Description : Set a keyframe at the current time.
 * Parameters :  pos - The position of the keyframe
 * Returns :     none
 */
void CAnimChannelEnable::SetKeyframe(bool enable)
{
    // Create a keyframe object
    KeyframeEnab keyframe;
    keyframe.SetFrame(GetTimeline()->GetCurrentFrame());
    keyframe.SetEnab(enable);

    switch(GetInsertAction())
    {
    case Append:
        mKeyframes.push_back(keyframe);
        mKeyframe1 = (int)mKeyframes.size() - 1;
        break;

    case Replace:
        mKeyframes[mKeyframe1] = keyframe;
        break;

    case Insert:
        mKeyframes.insert(mKeyframes.begin() + mKeyframe1 + 1, keyframe);
        mKeyframe1++;
        break;
    }
}

/*
 * Name :        CAnimChannelEnable::GetEnab()
 * Description : Get the current enable status.
 * Parameters :  none
 * Returns :     Current status
 */
bool CAnimChannelEnable::GetEnab(bool old)
{
    // If no keyframes we don't move anything
    if(mKeyframe1 < 0 && mKeyframe2 < 0)
        return old;

    // If we only have one keyframe, return its value
    if(mKeyframe2 < 0)
        return mKeyframes[mKeyframe1].GetEnab();

    // Same if either is the only keyframe
    if(mKeyframe1 < 0)
        return mKeyframes[mKeyframe2].GetEnab();

    //
    // We have two keyframes. Possibly edit the transparency of the drawable
    //
    bool enabFm = mKeyframes[mKeyframe1].GetEnab();

    return enabFm;
}

/*
 * Name :        CAnimChannelEnable::ClearKeyframe()
 * Description : Determines if a keyframe should be removed
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelEnable::ClearKeyframe()
{
	int currFrame = GetTimeline()->GetCurrentFrame();
	if (mKeyframe2 != -1 && currFrame == mKeyframes[mKeyframe2].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe2);
		if (mKeyframe2 < int(mKeyframes.size()))
			mKeyframe2++;
		else
			mKeyframe2 = -1;
	}
	if (mKeyframe1 != -1 && currFrame == mKeyframes[mKeyframe1].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe1);
		if (mKeyframes.size() > 0)
			mKeyframe1--;
		else
			mKeyframe1 = -1;
	}
}

/*
 * Name :        CAnimChannelEnable::EmptyChannel()
 * Description : Clears all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelEnable::EmptyChannel()
{
	int index = mKeyframes.size();
	while (index > 0)
	{
		mKeyframes.erase(mKeyframes.begin() + (index - 1));
		index--;
	}
    mKeyframe1 = -1;
    mKeyframe2 = -1;
}

