/*
 * Name:     CAnimChannelText.cpp
 *
 *
 * Class:    CAnimChannelText
 * Header:   CAnimChannelText.h
 * Created : Mar 21, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CAnimChannelText.h"
#include "../CTimeline.h"

/*
 * Name :        CAnimChannelText::~CAnimChannelText()
 * Description : Text channel status constructor
 */
CAnimChannelText::CAnimChannelText()
{
}

/*
 * Name :        CAnimChannelText::~CAnimChannelText()
 * Description : Text channel status destructor
 */
CAnimChannelText::~CAnimChannelText()
{
}

/*
 * Name :        CAnimChannelText::GetNumKeyframes()
 * Description : Obtain the number of keyframes. This is the size of the keyframe list.
 * Parameters :  none
 * Returns :     Number of keyframes.
 */
int CAnimChannelText::GetNumKeyframes()
{
    return (int)mKeyframes.size();
}

/*
 * Name :        CAnimChannelText::GetKeyframe()
 * Description : Get a keyframe by index from the list of keyframes.
 * Parameters :  frame - The frame index
 * Returns :     Pointer to keyframe at that index.
 */
CAnimChannel::Keyframe *CAnimChannelText::GetKeyframe(int frame)
{
    return &mKeyframes[frame];
}

/*
 * Name :        CAnimChannelText::SetKeyframe()
 * Description : Set a keyframe at the current time.
 * Parameters :  string - The contained string
 * Returns :     none
 */
void CAnimChannelText::SetKeyframe(std::wstring string)
{
    // Create a keyframe object
    KeyframeText keyframe;
    keyframe.SetFrame(GetTimeline()->GetCurrentFrame());
    keyframe.SetText(string);

    switch(GetInsertAction())
    {
    case Append:
        mKeyframes.push_back(keyframe);
        mKeyframe1 = (int)mKeyframes.size() - 1;
        break;

    case Replace:
        mKeyframes[mKeyframe1] = keyframe;
        break;

    case Insert:
        mKeyframes.insert(mKeyframes.begin() + mKeyframe1 + 1, keyframe);
        mKeyframe1++;
        break;
    }
}

/*
 * Name :        CAnimChannelText::GetAngle()
 * Description : Get the current text information, and facial animations
 * Parameters :  none
 * Returns :     The old text, and expression
 */
std::wstring CAnimChannelText::GetText(std::wstring old)
{
    // If no keyframes we don't move anything
    if(mKeyframe1 < 0 && mKeyframe2 < 0)
        return old;

    // If we only have one keyframe, return its value
    if(mKeyframe2 < 0)
        return mKeyframes[mKeyframe1].GetText();

    // Same if either is the only keyframe
    if(mKeyframe1 < 0)
        return mKeyframes[mKeyframe2].GetText();

    //
    // We have two keyframes. Just use the older one
    //
    std::wstring lowTxt = mKeyframes[mKeyframe1].GetText();
    return lowTxt;
}

/*
 * Name :        CAnimChannelText::ClearKeyframe()
 * Description : Determines if a keyframe should be removed
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelText::ClearKeyframe()
{
	int currFrame = GetTimeline()->GetCurrentFrame();
	if (mKeyframe2 != -1 && currFrame == mKeyframes[mKeyframe2].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe2);
		if (mKeyframe2 < int(mKeyframes.size()))
			mKeyframe2++;
		else
			mKeyframe2 = -1;
	}
	if (mKeyframe1 != -1 && currFrame == mKeyframes[mKeyframe1].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe1);
		if (mKeyframes.size() > 0)
			mKeyframe1--;
		else
			mKeyframe1 = -1;
	}
}

/*
 * Name :        CAnimChannelText::EmptyChannel()
 * Description : Clears all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelText::EmptyChannel()
{
	int index = mKeyframes.size();
	while (index > 0)
	{
		mKeyframes.erase(mKeyframes.begin() + (index - 1));
		index--;
	}
    mKeyframe1 = -1;
    mKeyframe2 = -1;
}
