/*
 * Name:     CDrawable.h
 *
 *
 * Class:    CDrawable
 * Impl:     CDrawable.cpp
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#pragma once
#ifndef CDRAWABLE_H_
#define CDRAWABLE_H_

#include "../Channel/CAnimChannelAngle.h"
#include "../Channel/CAnimChannelPos.h"
#include "../Channel/CAnimChannelEnable.h"
#include "../CTimeline.h"
#include <wx/xml/xml.h>
#include <list>

class CActor;
class CFrame;

/*
 * Class:    CDrawable
 * 
 */
class CDrawable
{
public:
	CDrawable(const std::wstring &name);
	virtual ~CDrawable();

	void Place(wxPoint offset, double rotate);
	void AddChild(CDrawable *child);
	void AddSChild(CDrawable *child);
	virtual void XmlSaveDrawable(wxXmlNode* actor);
	virtual void XmlLoad(std::wstring name, bool defaultParam, wxXmlNode *check);
	virtual void Draw(wxDC *dc) = 0;

	virtual bool HitTest(wxPoint pos) = 0;
	virtual bool IsMovable();
	virtual bool IsRotatable() {return true;}
	void Move(wxPoint delta);
	void TogEnabled() {mEnabled == false ? mEnabled = true : mEnabled = false;}
	virtual void TogBlink() {;}
	virtual void SetBlinkState(bool b) {;}

	virtual void SetPosition(wxPoint p);
	virtual wxPoint GetPosition() const {return mPosition;}
	void SetEnabled(bool e) {mEnabled = e;}
	bool GetEnabled() const {return mEnabled;}
	void SetName(const std::wstring &name) {mName = name;}
	std::wstring GetName() const {return mName;}
	virtual void SetRotation(double r);
	double GetRotation() const {return mRotation;}
	virtual void SetActor(CActor* a);
	CActor* GetActor() const {return mActor;}
	void SetParent(CDrawable* mommy) {mParent = mommy; mDefaultRot = mRotation; mDefaultPos = mPosition;}
	CDrawable* GetParent() const {return mParent;}
	virtual void SetText(const std::wstring &str) {return;}
    virtual void SetMirr(std::wstring check) {return;}
	virtual void SetTimeline(CTimeline *timeline);
	virtual void SetKeyframe();
	virtual void GetKeyframe();
	virtual void ClearKeyframe();
	virtual void ClearChannels();
	virtual bool HasDialog() {return false;}
	virtual void DispDiag(CFrame *frame) {;}
	virtual void SetDefault() {mRotation = mDefaultRot; mPosition = mDefaultPos;}

protected:
	wxPoint RotatePoint(wxPoint point, double angle);
	double mRotation;
	wxPoint mPosition;
	bool mEnabled;
	double mDefaultRot;
	wxPoint mDefaultPos;
	std::wstring mName;
	CActor* mActor;
    CAnimChannelEnable mChannelE;
    CAnimChannelAngle mChannelA;
    CAnimChannelPos mChannelP;

private:
	CDrawable();
	std::list<CDrawable *> mChildren;
	CDrawable* mParent;

protected:
	wxPoint mPlacedPosition;
	double mPlacedR;
};

#endif /* CDRAWABLE_H_ */
