/*
 * Name:     CHead.cpp
 *
 *
 * Class:    CHead
 * Header:   CHead.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "../CFrame.h"
#include "CHead.h"
#include "CHeadDlg.h"
#include "CNoRotationImageDrawable.h"
#include "CPolyDrawable.h"
#include <sstream>

/*
 * Name :        CHead::CHead()
 * Description : Head (ImagedDrawable) class constructor.
 */
//CHead::CHead()
//{
//}

/*
 * Name :        CHead::CHead()
 * Description : Constructor for an image-based drawable of a head.
 */

CHead::CHead(const std::wstring &name, const std::wstring &filename, int indicator) : CImageDrawable(name, filename)
{
	mClass = indicator-1;
	mFace = ID_Blank;
	mInitial = true;
	mBlinks = false;
	mCurrFace = ID_Blank;
	mRightEye = NULL;
	mRightEyebrow = NULL;
	mLeftEye = NULL;
	mLeftEyebrow = NULL;
	mBothEyes = NULL;

	// TODO Move each mClass into its own class CHaroldHead & CDanishHead
	// This make more sense if all eyes/eyebrows are the same no matter the head type
    if (mClass == Harold)
    {
		CNoRotationImageDrawable *leye = new CNoRotationImageDrawable(L"Left Eye", L"images/eye.png");
		leye->SetPosition(wxPoint(6, -36));
		this->AddSChild(leye);
	    mLeftEye = leye;

		CPolyDrawable *leyeb = new CPolyDrawable(L"Left Eyebrow");
		leyeb->SetColor(wxColour(0, 0, 0));
		leyeb->SetPosition(wxPoint(5, -4));
		leyeb->AddPoint(wxPoint(-6, 2));
		leyeb->AddPoint(wxPoint(12, 2));
		leyeb->AddPoint(wxPoint(12, -1));
		leyeb->AddPoint(wxPoint(-6, -1));
		leye->AddSChild(leyeb);
		mLeftEyebrow = leyeb;

		CNoRotationImageDrawable *reye = new CNoRotationImageDrawable(L"Right Eye", L"images/eye.png");
		reye->SetPosition(wxPoint(-24, -36));
		this->AddSChild(reye);
		mRightEye = reye;

		CPolyDrawable *reyeb = new CPolyDrawable(L"Right Eyebrow");
		reyeb->SetColor(wxColour(0, 0, 0));
		reyeb->SetPosition(wxPoint(12, -4));
		reyeb->AddPoint(wxPoint(-12, 2));
		reyeb->AddPoint(wxPoint(6, 2));
		reyeb->AddPoint(wxPoint(6, -1));
		reyeb->AddPoint(wxPoint(-12, -1));
		reye->AddSChild(reyeb);
		mRightEyebrow = reyeb;

		mDrawables.push_back(leye);
		mDrawables.push_back(leyeb);
		mDrawables.push_back(reye);
		mDrawables.push_back(reyeb);
	}
    else if (mClass == DanishCousin)
	{
		CNoRotationImageDrawable *eyes = new CNoRotationImageDrawable(L"Eyes", L"images/danish_cousin_eyes.png");
		eyes->SetCenter(wxPoint(56, 18));
		eyes->SetPosition(wxPoint(0, -30));
		this->AddSChild(eyes);
		mBothEyes = eyes;

		CPolyDrawable *reyeb = new CPolyDrawable(L"Right Eyebrow");
		reyeb->SetColor(wxColour(0, 0, 0));
		reyeb->SetPosition(wxPoint(-14, -18));
		reyeb->AddPoint(wxPoint(-16,2));
		reyeb->AddPoint(wxPoint(-12, 1));
		reyeb->AddPoint(wxPoint(6, 1));
		reyeb->AddPoint(wxPoint(6, -1));
		reyeb->AddPoint(wxPoint(-12, -1));
		reyeb->AddPoint(wxPoint(-14,4));
		eyes->AddSChild(reyeb);
		mRightEyebrow = reyeb;

		CPolyDrawable *leyeb = new CPolyDrawable(L"Left Eyebrow");
		leyeb->SetColor(wxColour(0, 0, 0));
		leyeb->SetPosition(wxPoint(10, -18));
		leyeb->AddPoint(wxPoint(16,2));
		leyeb->AddPoint(wxPoint(12, 1));
		leyeb->AddPoint(wxPoint(-6, 1));
		leyeb->AddPoint(wxPoint(-6, -1));
		leyeb->AddPoint(wxPoint(12, -1));
		leyeb->AddPoint(wxPoint(14,4));
		eyes->AddSChild(leyeb);
		mLeftEyebrow = leyeb;

		mDrawables.push_back(eyes);
		mDrawables.push_back(leyeb);
		mDrawables.push_back(reyeb);
	}

	mImage.LoadFile(filename.c_str());
	SetCenter(wxPoint(0, 0));
}

/*
 * Name :        CHead::~CHead()
 * Description : Head (ImagedDrawable) class destructor.
 */
CHead::~CHead()
{
}

/*
 * Name :        CHead::Draw()
 * Description : Draw the drawable image.
 * Parameters :  dc - Device context
 * Returns :     none
 */
void CHead::Draw(wxDC *dc)
{
    // Don't draw if not enabled
    if(!mEnabled)
        return;

	// Draw the base for the head
	CImageDrawable::Draw(dc);

	// Draw other parts of the head
    for(std::list<CDrawable *>::iterator d=mDrawables.begin(); d!=mDrawables.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->Draw(dc);
    }
}

/*
 * Name :        CHead::DispDiag()
 * Description : Calls CHeadDlg
 * Parameters :  The source frame
 * Returns :     none
 */
void CHead::DispDiag(CFrame *frame)
{
    CHeadDlg dlg(frame, this, frame->GetEditView());
    if(dlg.ShowModal() == wxOK)
    {
    	GetActor()->GetDocument()->UpdateObservers();
    }
}

/*
 * Name :        CHead::SetExpression()
 * Description : Changes the face accordingly
 * Parameters :  The requested expression ID
 * Returns :     none
 */
void CHead::SetExpression(int face)
{
	mFace = (Ids)face;
	if (mInitial)
	{
		mTempL = mLeftEyebrow->GetPosition();
		mTempR = mRightEyebrow->GetPosition();
		mInitial = false;
	}
	switch(mFace)
	{
	case (ID_Sad):
	{
		mRightEyebrow->SetPosition(mTempR);
		mLeftEyebrow->SetPosition(mTempL);
		mRightEyebrow->SetRotation(0.33);
		mLeftEyebrow->SetRotation(-0.33);
		mCurrFace = ID_Sad;
		break;
	}
	case (ID_Happy):
	{
		mLeftEyebrow->SetPosition(wxPoint(mTempL.x, (mTempL.y - 3)));
		mRightEyebrow->SetPosition(wxPoint(mTempR.x, (mTempR.y - 3)));
		mRightEyebrow->SetRotation(0.1);
		mLeftEyebrow->SetRotation(-0.1);
		mCurrFace = ID_Happy;
		break;
	}
	case (ID_Angry):
	{
		mRightEyebrow->SetPosition(mTempR);
		mLeftEyebrow->SetPosition(mTempL);
		mRightEyebrow->SetRotation(-0.5);
		mLeftEyebrow->SetRotation(0.5);
		mCurrFace = ID_Angry;
		break;
	}
	case (ID_Confused):
	{
		mRightEyebrow->SetPosition(mTempR);
		mRightEyebrow->SetRotation(0.33);
		mLeftEyebrow->SetPosition(wxPoint(mTempL.x, (mTempL.y + 3)));
		mLeftEyebrow->SetRotation(0);
		mCurrFace = ID_Confused;
		break;
	}
	case (ID_Blank):
	{
		mRightEyebrow->SetRotation(0);
		mLeftEyebrow->SetRotation(0);
		mRightEyebrow->SetPosition(mTempR);
		mLeftEyebrow->SetPosition(mTempL);
		mCurrFace = ID_Blank;
		break;
	}
	case (ID_Blinking):
	{
		TogBlinks();
		if (mClass == Harold)
		{
			mLeftEye->SetBlinkState(mBlinks);
			mRightEye->SetBlinkState(mBlinks);
		}
		else if (mClass == DanishCousin)
		{
			mBothEyes->SetBlinkState(mBlinks);
		}
		break;
	}
	default:
	{
		//std::cerr << "ERROR" << std::endl;
	}
	}
}

/*
 * Name :        CHead::SetTimeline()
 * Description : Set the timeline for all channels.
 * Parameters :  timeline - The timeline class.
 * Returns :     none
 */
void CHead::SetTimeline(CTimeline *timeline)
{
    timeline->AddChannel(&mChannelExpression);
    CDrawable::SetTimeline(timeline);
}

/*
 * Name :        CHead::SetKeyframe()
 * Description : Set the keyframe based on the current status.
 * Parameters :  none
 * Returns :     none
 */
void CHead::SetKeyframe()
{
	int temp = (10 * mCurrFace) + mBlinks;
	std::wstringstream str;
	str << temp;
	mChannelExpression.SetKeyframe(str.str());
    CDrawable::SetKeyframe();
}

/*
 * Name :        CHead::GetKeyframe()
 * Description : Get the current channel from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CHead::GetKeyframe()
{
	if (mChannelExpression.GetNumKeyframes() > 0)
	{
		int num = (10 * mCurrFace) + mBlinks;
		std::wstringstream old;
		old << num;
		wxString str = mChannelExpression.GetText(old.str());
		int temp = wxAtoi(str);
		SetExpression(temp/10);
		if (temp%10 == 1)
		{
			mBlinks = true;
		}
		else
		{
			mBlinks = false;
		}
		if (mClass == Harold)
		{
			mLeftEye->SetBlinkState(temp%10);
			mRightEye->SetBlinkState(temp%10);
		}
		else if (mClass == DanishCousin)
		{
			mBothEyes->SetBlinkState(temp%10);
		}
	}
	CDrawable::GetKeyframe();
}

/*
 * Name :        CHead::ClearKeyframe()
 * Description : Clear the current frame from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CHead::ClearKeyframe()
{
	mChannelExpression.ClearKeyframe();
	CDrawable::ClearKeyframe();
}

/*
 * Name :        CHead::ClearChannels()
 * Description : Clear the channel of all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CHead::ClearChannels()
{
	mChannelExpression.EmptyChannel();
	CDrawable::ClearChannels();
}

/*
 * Name :        CHead::XmlSaveDrawable()
 * Description : Saves information for a drawable
 * Parameters :  The actor it belongs to
 * Returns :     none
 */
void CHead::XmlSaveDrawable(wxXmlNode* actor)
{
	// Convert any spaces to an underscore
	std::wstring name = GetName();
	std::wstring spc(L" ");
	std::wstring und(L"_");
	for (unsigned int i = 0; i < name.length(); i++)
	{
		if (name[i] == spc)
		{
			name[i] =und[0];
		}
	}

	// Create a node for this drawable item
	wxXmlNode *drawable = new wxXmlNode(wxXML_ELEMENT_NODE, name);

	// Add a position if applicable to save space
	if (IsMovable())
	{
		//wxXmlNode *position = new wxXmlNode(wxXML_ELEMENT_NODE, L"position");
		wxPoint tempPos = mChannelP.GetPos(mPosition);
		drawable->AddProperty(L"x", wxString::Format(L"%d", tempPos.x));
		drawable->AddProperty(L"y", wxString::Format(L"%d", tempPos.y));
		//drawable->AddChild(position);
	}

	// Add angle information (default to zero?)
	//wxXmlNode *angle = new wxXmlNode(wxXML_ELEMENT_NODE, L"angle");
	double tempAng = mChannelA.GetAngle();
	// Only save two decimal points for angles since the rest are 0
	drawable->AddProperty(L"angle", wxString::Format(L"%.2f", tempAng));
	//drawable->AddChild(angle);
	bool tempEnab = mChannelE.GetEnab(mEnabled);
	drawable->AddProperty(L"on", wxString::Format(L"%d", tempEnab));
	int num = (10 * mCurrFace) + mBlinks;
	std::wstringstream old;
	old << num;
	wxString str = mChannelExpression.GetText(old.str());
	drawable->AddProperty(L"face", str);

	actor->AddChild(drawable);
}

/*
 * Name :        CHead::XmlLoad()
 * Description : Loads information about the drawable from file
 * Parameters :  the name of the drawable to be loading
 * Returns :     none
 */
void CHead::XmlLoad(std::wstring name, bool defaultParam, wxXmlNode *check)
{
	if (name == mName)
	{
		wxString rot = check->GetPropVal(L"angle", L"");
		wxString enb = check->GetPropVal(L"on", L"");
		SetEnabled(wcstol(enb, NULL, 2));
		if (IsRotatable())
		{
			SetRotation(wcstod(rot, NULL));
		}
		if (IsMovable())
		{
			wxString X = check->GetPropVal(L"x", L"");
			wxString Y = check->GetPropVal(L"y", L"");
			SetPosition(wxPoint(wcstol(X, NULL, 10), wcstol(Y, NULL, 10)));
		}
		wxString face = check->GetPropVal(L"face", L"");
		int tempFace = wcstol(face,NULL,10);
		SetExpression(tempFace / 10);
		if (tempFace % 10 != mBlinks)
		{
			SetExpression(ID_Blinking);
		}
		if (defaultParam)
		{
			mDefaultPos = GetPosition();
			mDefaultRot = GetRotation();
		}
	}
}

