/*
 * CHeadDlg.cpp
 *
 *  Created on: Mar 25, 2010
 *      Author: aonokenj
 */

#include "../wx/prec.h"
#include "CHeadDlg.h"

BEGIN_EVENT_TABLE(CHeadDlg, wxDialog)
    EVT_BUTTON(ID_Ok, CHeadDlg::OnOkButton)
    EVT_BUTTON(ID_Cancel, CHeadDlg::OnCancelButton)
    EVT_BUTTON(ID_Happy, CHeadDlg::SetHappyFace)
    EVT_BUTTON(ID_Angry, CHeadDlg::SetAngryFace)
    EVT_BUTTON(ID_Confused, CHeadDlg::SetConfusedFace)
    EVT_BUTTON(ID_Blank, CHeadDlg::SetBlankFace)
    EVT_BUTTON(ID_Sad, CHeadDlg::SetSadFace)
    EVT_BUTTON(ID_Blinking, CHeadDlg::SetBlinkingFace)
END_EVENT_TABLE()

/*
 * Name :        CHeadDlg::CHeadDlg()
 * Description : Constructor for a head dialog box, creates the dialog box.
 * Parameters :  parent - a parent window for the box
 *               head - The haed we are editing
 * Returns :     none
 */
CHeadDlg::CHeadDlg(wxWindow *parent, CHead *head, CViewEdit *showHere) :
    wxDialog(parent, wxID_ANY, L"Facial Expressions", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE),
    mHead(head)
{
	mViewEdit = showHere;
	mFace = (Ids)mHead->GetExpression();
    // Create a vertical sizer so we have a top part of the box
    // for the options and a bottom part for the Ok and Cancel buttons
    wxBoxSizer* vSizer;
    vSizer = new wxBoxSizer( wxVERTICAL );

    wxStaticBoxSizer *boxSizer = new wxStaticBoxSizer(wxVERTICAL, this, L"Choices:");
    boxSizer->AddSpacer(125);
    vSizer->Add(boxSizer, 1, wxALL | wxEXPAND);

    mHappy = new wxButton(this, ID_Happy, wxT("Happy"),  wxPoint(100, 60),
    		              wxSize(60, 30));
    mAngry = new wxButton(this, ID_Angry, wxT("Angry"),  wxPoint(25, 60),
    		              wxSize(60, 30));
    mConfused = new wxButton(this, ID_Confused, wxT("Confused"),  wxPoint(100, 20),
    		              wxSize(60, 30));
    mBlank = new wxButton(this, ID_Blank, wxT("Blank"),  wxPoint(25, 20),
    		              wxSize(60, 30));
    mBlink = new wxButton(this, ID_Sad, wxT("Sad"), wxPoint(25, 100),
                          wxSize(60,30));
    mBlink = new wxButton(this, ID_Blinking, wxT("Blink"), wxPoint(100, 100),
                          wxSize(60,30));

    // Create a horizontal sizer that will hold the Ok and Cancel buttons
    // and add it to the vertical sizer.
    wxBoxSizer *hSizer;
    hSizer = new wxBoxSizer(wxHORIZONTAL);
    vSizer->Add(hSizer, 0, wxALL | wxEXPAND);

    // Create an Ok button and add it to the horizontal sizer.
    mOkButton = new wxButton( this, ID_Ok, L"Ok", wxDefaultPosition, wxDefaultSize, 0 );
    hSizer->Add(mOkButton, 1, wxALL | wxEXPAND);
    mOkButton->SetDefault();

    // Create a Cancel button and add to the horizontal sizer
    mCancelButton = new wxButton( this, ID_Cancel, L"Cancel", wxDefaultPosition, wxDefaultSize, 0 );
    hSizer->Add(mCancelButton, 1, wxALL | wxEXPAND);

    // Set the top level sizer and force the window to match the size
    this->SetSizer( vSizer );
    this->Layout();
    vSizer->Fit( this );
}

/*
 * Name :        CHeadDlg::~CHeadDlg()
 * Description : Destructor for the head dialog box
 */
CHeadDlg::~CHeadDlg()
{
}

/*
 * Name :        CHeadDlg::OnOkButton()
 * Description : Handle an Ok button press.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CHeadDlg::OnOkButton(wxCommandEvent &event)
{
    EndModal(wxOK);
}


/*
 * Name :        CHeadDlg::OnCancelButton()
 * Description : Handle an Cancel button press.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CHeadDlg::OnCancelButton(wxCommandEvent &event)
{
	// Void any changes made
	mHead->SetExpression(mFace);
	mViewEdit->UpdateObserver();
    EndModal(wxCANCEL);
}

/*
 * Name :        CHeadDlg::SetFace()
 * Description : Changes the face for the head
 * Parameters :  event - The command event
 * Returns :     none
 */
void CHeadDlg::SetFace(Ids face)
{
	mHead->SetExpression(face);
	mViewEdit->UpdateObserver();
}
