/*
 * Name:     CImageDrawable.cpp
 *
 *
 * Class:    CImageDrawable
 * Header:   CImageDrawable.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CImageDrawable.h"

/*
 * Name :        CImageDrawable::CImageDrawable()
 * Description : ImageDrawable class constructor.
 */
//CImageDrawable::CImageDrawable()
//{
//}

/*
 * Name :        CImageDrawable::CImageDrawable()
 * Description : Constructor for an image-based drawable.
 */

CImageDrawable::CImageDrawable(const std::wstring &name, const std::wstring &filename) : CDrawable(name)
{
    mImage.LoadFile(filename.c_str());
    mCenter = wxPoint(0, 0);
    mRotatedBitmapValid = false;
}

/*
 * Name :        CImageDrawable::~CImageDrawable()
 * Description : ImageDrawable class destructor.
 */
CImageDrawable::~CImageDrawable()
{
}

/*
 * Name :        CImageDrawable::Draw()
 * Description : Draw the drawable image.
 * Parameters :  dc - Device context
 * Returns :     none
 */
void CImageDrawable::Draw(wxDC *dc)
{
    // Don't draw if not enabled
    if(!mEnabled)
        return;

    // See if the bitmap we have is invalid...
    if(!mRotatedBitmapValid || mBitmapR != mPlacedR)
    {
        // Create a new rotated image
        wxPoint offsetAfterRotation;
        mRotatedImage = mImage.Rotate(mPlacedR, mCenter, true, &offsetAfterRotation);

        // Compute the location of the center point after rotation
        mBitmapCenter = mCenter - offsetAfterRotation;

        // And create a bitmap from the rotated image.
        mRotatedBitmap = wxBitmap(mRotatedImage);
        mBitmapR = mPlacedR;

        mRotatedBitmapValid = true;
    }

    // Draw it relative to the new center of the bitmap
    dc->DrawBitmap(mRotatedBitmap, mPlacedPosition.x - mBitmapCenter.x, mPlacedPosition.y - mBitmapCenter.y, true);
}

/*
 * Name :        CImageDrawable::HitTest()
 * Description : Test to see if we hit this object with a mouse.
 * Parameters :  pos - x,y postion
 * Returns :     true if hit.
 */
bool CImageDrawable::HitTest(wxPoint pos)
{
    // Make x and y relative to the top-left corner of the rotated image
    pos -= mPlacedPosition - mBitmapCenter;
    int x = pos.x;
    int y = pos.y;

    if(x < 0 || y < 0 || x >= mRotatedImage.GetWidth() || y >= mRotatedImage.GetHeight())
    {
        // We are outside the image
        return false;
    }

    return !mRotatedImage.HasAlpha() || mRotatedImage.GetAlpha(x, y) > 0;
}

/*
 * Name :        CImageDrawable::SetCenter()
 * Description : Set the center
 * Parameters :  The new center
 * Returns :     none
 */
void CImageDrawable::SetCenter(wxPoint c)
{
	mCenter = c;
}
