/*
 * Name:     CNoRotationImageDrawable.cpp
 *
 *
 * Class:    CNoRotationImageDrawable
 * Header:   CNoRotationImageDrawable.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CNoRotationImageDrawable.h"

/*
 * Name :        CNoRotationImageDrawable::CNoRotationImageDrawable()
 * Description : Image drawable (no rotation) class constructor.
 */
//CNoRotationImageDrawable::CNoRotationImageDrawable()
//{
//}

/*
 * Name :        CNoRotationImageDrawable::CNoRotationImageDrawable()
 * Description : Constructor for an image drawable (no rotation)
 */

CNoRotationImageDrawable::CNoRotationImageDrawable(const std::wstring &name, const std::wstring &filename) : CImageDrawable(name, filename)
{
    mImage.LoadFile(filename.c_str());
    SetCenter(wxPoint(0, 0));

    mBlinking = false;

    // Determines how to blink
    if (name == L"Eyes")
    {
		mPoints.push_back(wxPoint(-25,10));
		mPoints.push_back(wxPoint(0,5));
		mPoints.push_back(wxPoint(25,10));
		mPoints.push_back(wxPoint(27,8));
		mPoints.push_back(wxPoint(5,0));
		mPoints.push_back(wxPoint(27,-8));
		mPoints.push_back(wxPoint(25,-10));
		mPoints.push_back(wxPoint(0,-5));
		mPoints.push_back(wxPoint(-25,-10));
		mPoints.push_back(wxPoint(-27,-8));
		mPoints.push_back(wxPoint(-5, 0));
    }
    else // This will also be applied to the vest...
    {
		mPoints.push_back(wxPoint(-1,19));
		mPoints.push_back(wxPoint(9,12));
		mPoints.push_back(wxPoint(19,19));
		mPoints.push_back(wxPoint(21,14));
		mPoints.push_back(wxPoint(12,9));
		mPoints.push_back(wxPoint(21,4));
		mPoints.push_back(wxPoint(19,-1));
		mPoints.push_back(wxPoint(9,6));
		mPoints.push_back(wxPoint(-1,-1));
		mPoints.push_back(wxPoint(-3,4));
		mPoints.push_back(wxPoint(6, 9));
    }
}

/*
 * Name :        CNoRotationImageDrawable::~CNoRotationImageDrawable()
 * Description : Image drawable (no rotation) class destructor.
 */
CNoRotationImageDrawable::~CNoRotationImageDrawable()
{
}

/*
 * Name :        CNoRotationImageDrawable::Draw()
 * Description : Draw the drawable image.
 * Parameters :  dc - Device context
 * Returns :     none
 */
void CNoRotationImageDrawable::Draw(wxDC *dc)
{
    // Don't draw if not enabled
    if(!mEnabled)
        return;

    if (!mBlinking)
    {
		// See if the bitmap we have is invalid...
		if(!mRotatedBitmapValid || mBitmapR != mPlacedR)
		{
			// Create a new rotated image
			wxPoint offsetAfterRotation;
			mRotatedImage = mImage.Rotate(mPlacedR, mCenter, true, &offsetAfterRotation);

			// Compute the location of the center point after rotation
			mBitmapCenter = mCenter - offsetAfterRotation;

			// And create a bitmap from the rotated image.
			mRotatedBitmap = wxBitmap(mRotatedImage);
			mBitmapR = mPlacedR;

			mRotatedBitmapValid = true;
		}

		// Draw it relative to the new center of the bitmap
		dc->DrawBitmap(mRotatedBitmap, mPlacedPosition.x - mBitmapCenter.x, mPlacedPosition.y - mBitmapCenter.y, true);
    }

    else
    {
    	wxBrush brush(*wxBLACK);
        dc->SetBrush(brush);

        wxPen pen(*wxBLACK, 0);
        dc->SetPen(pen);

        // Transform the points
        std::vector<wxPoint> points;
        for(std::vector<wxPoint>::iterator p=mPoints.begin(); p!=mPoints.end(); p++)
        {
            points.push_back(RotatePoint(*p, mPlacedR) + mPlacedPosition);
        }

        dc->DrawPolygon((int)points.size(), &points[0]);

        dc->SetBrush(wxNullBrush);
        dc->SetPen(wxNullPen);
    }
}
