/*
 * Name:     CTextBubble.h
 * This class implements a basic text bubble control.
 *
 * Class:    CTextBubble
 * Impl:     CTextBubble.cpp
 * Created : Feb 28, 2010
 * Author :  Charles B. Owen
 */

#pragma once
#ifndef CTEXTBUBBLE_H_
#define CTEXTBUBBLE_H_

#ifndef _WX_WX_H_
#include <wx/wx.h>
#endif

#include <string>
#include <vector>


/*
 * Class:    CTextBubble
 * 
 */
class CTextBubble
{
public:
    CTextBubble();
    virtual ~CTextBubble();

    void DrawMe(wxDC *dc);
    bool IsPointInside(int x, int y);

    std::wstring GetText() const {return mText;}
    void SetText(const std::wstring &str);

    bool GetMirror() const {return mMirror;}
    void SetMirror(bool m) {mMirror = m;  mNewString = true;}

    void SetBubblePosition(int x, int y) {mPosition = wxPoint(x, y);}
    int GetX() const {return mPosition.x;}
    int GetY() const {return mPosition.y;}

private:
    void NewString(wxDC *dc);

    bool     mMirror;   // Should we mirror the text box?
    std::wstring mText; // The unadulterated text
    wxPoint  mPosition; // Position for the tip of the text bubble
    wxFont   mFont;

    bool     mNewString;

    wxImage  mImage;
    wxImage  mImageScaled;
    wxBitmap mBitmap;

    // The array of wxString objects that holds the split text
    std::vector<wxString> mSplitText;
};

#endif /* CTEXTBUBBLE_H_ */
