/*
 * Name:     CDanishCousinFactory.cpp
 *
 *
 * Class:    CDanishCousinFactory
 * Header:   CDanishCousinFactory.h
 * Created : Feb 27, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CDanishCousinFactory.h"
#include "../Drawable/CPolyDrawable.h"
#include "../Drawable/CImageDrawable.h"
#include "../Drawable/CHead.h"
#include "../Drawable/CNoRotationImageDrawable.h"
#include "../Drawable/CTextBubbleDrawable.h"

using namespace std;

/*
 * Name :        CDanishCousinFactory::CDanishCousinFactory()
 * Description : DanishCousinFactory constructor.
 */
CDanishCousinFactory::CDanishCousinFactory()
{
}

/*
 * Name :        CDanishCousinFactory::~CDanishCousinFactory()
 * Description : DanishCousinFactory destructor.
 */
CDanishCousinFactory::~CDanishCousinFactory()
{
}

/*
 * Name :        CDanishCousinFactory::Create()
 * Description : Create a 'Danish Cousin' actor
 * Parameters :  none
 * Returns :     pointer to a CActor object populated with 'Danish Cousin'
 */
CActor * CDanishCousinFactory::Create()
{
    CActor *actor = new CActor(L"Danish Cousin");

    CImageDrawable *shirt = new CImageDrawable(L"Shirt", L"images/danish_cousin_coat.png");
    shirt->SetCenter(wxPoint(46, 138));
    shirt->SetPosition(wxPoint(0, -114));
    actor->SetRoot(shirt);

    CImageDrawable *headb = new CImageDrawable(L"Head Bottom", L"images/danish_cousin_headb.png");
    headb->SetCenter(wxPoint(43,31));
    headb->SetPosition(wxPoint(0, -130));
    shirt->AddChild(headb);

    CHead *headt = new CHead(L"Head Top", L"images/danish_cousin_headt.png", DanishCousin);
    headt->SetCenter(wxPoint(76, 109));
    headt->SetPosition(wxPoint(0, -31));
    headb->AddChild(headt);

    CPolyDrawable *larm = new CPolyDrawable(L"Left Arm");
    larm->SetColor(wxColour(20, 25, 26));
    larm->SetPosition(wxPoint(46, -130));
    larm->AddPoint(wxPoint(-8, -7));
    larm->AddPoint(wxPoint(-8, 44));
    larm->AddPoint(wxPoint(8, 44));
    larm->AddPoint(wxPoint(8, -7));
    shirt->AddChild(larm);

    CPolyDrawable *rarm = new CPolyDrawable(L"Right Arm");
    larm->SetColor(wxColour(20, 25, 26));
    rarm->SetPosition(wxPoint(-45, -130));
    rarm->AddPoint(wxPoint(-8, -7));
    rarm->AddPoint(wxPoint(-8, 44));
    rarm->AddPoint(wxPoint(8, 44));
    rarm->AddPoint(wxPoint(8, -7));
    shirt->AddChild(rarm);

    CPolyDrawable *lfarm = new CPolyDrawable(L"Left Forearm");
    larm->SetColor(wxColour(20, 25, 26));
    lfarm->SetPosition(wxPoint(0, 44));
    lfarm->AddPoint(wxPoint(-8, -8));
    lfarm->AddPoint(wxPoint(-8, 52));
    lfarm->AddPoint(wxPoint(8, 52));
    lfarm->AddPoint(wxPoint(8, -8));
    larm->AddChild(lfarm);

    CPolyDrawable *rfarm = new CPolyDrawable(L"Right Forearm");
    larm->SetColor(wxColour(20, 25, 26));
    rfarm->SetPosition(wxPoint(0, 44));
    rfarm->AddPoint(wxPoint(-8, -8));
    rfarm->AddPoint(wxPoint(-8, 52));
    rfarm->AddPoint(wxPoint(8, 52));
    rfarm->AddPoint(wxPoint(8, -8));
    rarm->AddChild(rfarm);

    CPolyDrawable *lhand = new CPolyDrawable(L"Left Hand");
    lhand->SetColor(wxColour(210, 160, 118));
    lhand->SetPosition(wxPoint(0, 52));
    lhand->AddPoint(wxPoint(-12, -9));
    lhand->AddPoint(wxPoint(-12, 11));
    lhand->AddPoint(wxPoint(11, 11));
    lhand->AddPoint(wxPoint(11, -9));
    lfarm->AddChild(lhand);

    CPolyDrawable *rhand = new CPolyDrawable(L"Right Hand");
    rhand->SetColor(wxColour(210, 160, 118));
    rhand->SetPosition(wxPoint(0, 52));
    rhand->AddPoint(wxPoint(-12, -9));
    rhand->AddPoint(wxPoint(-12, 11));
    rhand->AddPoint(wxPoint(11, 11));
    rhand->AddPoint(wxPoint(11, -9));
    rfarm->AddChild(rhand);

    CPolyDrawable *lleg = new CPolyDrawable(L"Left Leg");
    lleg->SetColor(wxColour(62,70,106));
    lleg->SetPosition(wxPoint(25,0));
    lleg->AddPoint(wxPoint(-13, -12));
    lleg->AddPoint(wxPoint(-13, 52));
    lleg->AddPoint(wxPoint(20, 52));
    lleg->AddPoint(wxPoint(13, -12));
    shirt->AddChild(lleg);

    CPolyDrawable *rleg = new CPolyDrawable(L"Right Leg");
    rleg->SetColor(wxColour(62,70,106));
    rleg->SetPosition(wxPoint(-25,0));
    rleg->AddPoint(wxPoint(-13, -12));
    rleg->AddPoint(wxPoint(-20, 52));
    rleg->AddPoint(wxPoint(13, 52));
    rleg->AddPoint(wxPoint(13, -12));
    shirt->AddChild(rleg);

    CPolyDrawable *lshin = new CPolyDrawable(L"Left Shin");
    lshin->SetColor(wxColour(62,70,106));
    lshin->SetPosition(wxPoint(0,50));
    lshin->AddPoint(wxPoint(-13, -14));
    lshin->AddPoint(wxPoint(-13, 55));
    lshin->AddPoint(wxPoint(25, 55));
    lshin->AddPoint(wxPoint(18, -13));
    lleg->AddChild(lshin);

    CPolyDrawable *rshin = new CPolyDrawable(L"Right Shin");
    rshin->SetColor(wxColour(62,70,106));
    rshin->SetPosition(wxPoint(0,50));
    rshin->AddPoint(wxPoint(-18, -13));
    rshin->AddPoint(wxPoint(-25, 55));
    rshin->AddPoint(wxPoint(13, 55));
    rshin->AddPoint(wxPoint(13, -14));
    rleg->AddChild(rshin);

    CPolyDrawable *lfoot = new CPolyDrawable(L"Left Foot");
    lfoot->SetColor(wxColour(0,0,0));
    lfoot->SetPosition(wxPoint(-6,50));
    lfoot->AddPoint(wxPoint(36, -1));
    lfoot->AddPoint(wxPoint(44, 15));
    lfoot->AddPoint(wxPoint(-6, 15));
    lfoot->AddPoint(wxPoint(-6, -1));
    lshin->AddChild(lfoot);

    CPolyDrawable *rfoot = new CPolyDrawable(L"Right Foot");
    rfoot->SetColor(wxColour(0,0,0));
    rfoot->SetPosition(wxPoint(6,50));
    rfoot->AddPoint(wxPoint(6, -1));
    rfoot->AddPoint(wxPoint(6, 15));
    rfoot->AddPoint(wxPoint(-44, 15));
    rfoot->AddPoint(wxPoint(-36, -1));
    rshin->AddChild(rfoot);

    CTextBubbleDrawable *text = new CTextBubbleDrawable(L"Text Bubble");
    text->SetPosition(wxPoint(55,-90));
    text->SetText(std::wstring(L"Hi, I'm Danish !"));
    text->FlipMirror();
    headt->AddChild(text);

    actor->AddDrawable(rleg);
    actor->AddDrawable(rfoot);
    actor->AddDrawable(rshin);
    actor->AddDrawable(lleg);
    actor->AddDrawable(lfoot);
    actor->AddDrawable(lshin);
    actor->AddDrawable(shirt);
    actor->AddDrawable(headb);
    actor->AddDrawable(headt);
    actor->AddDrawable(rarm);
    actor->AddDrawable(larm);
    actor->AddDrawable(rfarm);
    actor->AddDrawable(lfarm);
    actor->AddDrawable(rhand);
    actor->AddDrawable(lhand);
    actor->AddDrawable(text);

    return actor;
}

