/*
 * Name:     CDocumentFactory.cpp
 *
 *
 * Class:    CDocumentFactory
 * Header:   CDocumentFactory.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CDocumentFactory.h"
#include "CHaroldFactory.h"
#include "CDanishCousinFactory.h"
#include "../Drawable/CImageDrawable.h"

/*
 * Name :        CDocumentFactory::CDocumentFactory()
 * Description : Document factory class constructor.
 */
CDocumentFactory::CDocumentFactory()
{
}

/*
 * Name :        CDocumentFactory::CDocumentFactory()
 * Description : Document factory class destructor.
 */
CDocumentFactory::~CDocumentFactory()
{
}

/*
 * Name :        CDocumentFactory::Create()
 * Description : Creates a new document object
 * Parameters :  none
 * Returns :     A pointer to the new document
 */
CDocument *CDocumentFactory::Create()
{
    CDocument *document = new CDocument();

    // Create the background and add it
    CActor *background = new CActor(L"Background");
    background->SetClickable(false);
    background->SetPosition(wxPoint(-100, 0));
    CImageDrawable *backgroundI = new CImageDrawable(L"Background", L"images/Background.png");
    background->AddDrawable(backgroundI);
    background->SetRoot(backgroundI);
    document->AddActor(background);

    background->SetHDrawable(background->GetRoot());

    // Create and add Harold
    CHaroldFactory factory;
    CActor *harold = factory.Create();

    // This is where Harold will start out.
    harold->SetPosition(wxPoint(360, 500));

    document->AddActor(harold);

	harold->SetHDrawable(harold->GetRoot());

    // Now lets add his Danish cousin
    CDanishCousinFactory ship;
    CActor *cousin = ship.Create();

    // And set him to the right of Harold
    cousin->SetPosition(wxPoint(550,500));

    document->AddActor(cousin);

	cousin->SetHDrawable(cousin->GetRoot());

    return document;
}
