/*
 * Name:     CParticle.cpp
 *
 *
 * Class:    CParticle
 * Header:   CParticle.h
 * Created : Apr 4, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CParticle.h"

/*
 * Name :        CParticle::CParticle()
 * Description : Particle class constructor
 */
CParticle::CParticle(int framerate)
{
	mWeight = framerate / 30.0;
	double randX = (-100 + rand() % 1000);
	double randY = (0 - rand() % 10);
	double randXdel = ((rand() % 1000) / 1000.0) / mWeight;
	double randYdel = ((rand() % 2500) / 1000.0) / mWeight;
	mPosition = wxRealPoint(randX,randY);
	mVelocity = wxRealPoint(randXdel,randYdel);
	mSway = true;
}

/*
 * Name :        CParticle::~CParticle()
 * Description : Particle class constructor
 */
CParticle::~CParticle()
{
}

/*
 * Name :        CParticle::Update()
 * Description : Update the position of the particle
 * Parameters :  none
 * Returns :     none
 */
void CParticle::Update()
{
	double temp = mVelocity.y;
	// Once in a while a particle should move up;
	if (rand() % 100 == 0)
	{
		temp = (-1) * temp;
	}
	// Sometimes the velocity should change (only increase)
	if (rand() % 20 == 0)
	{
		double randXdel = ((rand() % 1000) / 1000.0) / mWeight;
		double randYdel = ((rand() % 2500) / 1000.0) / mWeight;
		if (randYdel > mVelocity.y && randXdel > mVelocity.x)
			mVelocity = wxRealPoint(randXdel,randYdel);
	}
	mSway = (rand() % 10 == 0);
	if (mSway)
	{
		mPosition = wxRealPoint((mPosition.x + mVelocity.x),
		                        (mPosition.y + mVelocity.y));
	}
	// Don't sway (move in opposite x direction)
	else
	{
		mPosition = wxRealPoint((mPosition.x - mVelocity.x),
		                        (mPosition.y + mVelocity.y));
	}
}

/*
 * Name :        CParticle::Reset()
 * Description : Reset the stats of the particle
 * Parameters :  none
 * Returns :     none
 */
void CParticle::Reset()
{
	double randX = (-100 + rand() % 1000);
	double randY = (0 - rand() % 10);
	double randXdel = ((rand() % 1000) / 1000.0) / mWeight;
	double randYdel = ((rand() % 2500) / 1000.0) / mWeight;
	mPosition = wxRealPoint(randX,randY);
	mVelocity = wxRealPoint(randXdel,randYdel);
	mSway = true;
}
