/*
 * Name:     CParticlePool.cpp
 *
 *
 * Class:    CParticlePool
 * Header:   CParticlePool.h
 * Created : Apr 4, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CParticlePool.h"

/*
 * Name :        CParticlePool::CParticlePool()
 * Description : Particle Pool class constructor
 */
CParticlePool::CParticlePool()
{
}

/*
 * Name :        CParticlePool::~CParticlePool()
 * Description : Particle Pool class constructor
 */
CParticlePool::~CParticlePool()
{
}

/*
 * Name :        CParticlePool::GetParticle()
 * Description : Retrieve a particle from the front of the list
 * Parameters :  none
 * Returns :     A pointer to the particle
 */
CParticle* CParticlePool::GetParticle()
{
	return RemoveParticle();
}

/*
 * Name :        CParticlePool::AppendParticle()
 * Description : Append a particle to the end of the list
 * Parameters :  Particle to be added
 * Returns :     none
 */
void CParticlePool::AppendParticle(CParticle* p)
{
	AddParticle(p);
}

/*
 * Name :        CParticlePool::UpdatePosition()
 * Description : Update the position of a particle
 * Parameters :  none
 * Returns :     none
 */
void CParticlePool::UpdatePosition(CParticlePool* target)
{
	SLLNode *current, *prev;
	current = begin_ptr;
	prev = NULL;

	while (current != NULL)
	{
		current->mParticle->Update();
		if (current->mParticle->GetHeight() > 606 && prev != NULL)
		{
			// Remove current from the link
			prev->mNext = current->mNext;
			current->mParticle->Reset();
			target->AppendParticle(current->mParticle);
		}
		prev = current;
		current = current->mNext;
	}
}

/*
 * Name :        CParticlePool::DrawParticles()
 * Description : Draw any particles in the pool
 * Parameters :  dc - context to draw to
 * Returns :     none
 */
void CParticlePool::DrawParticles(wxDC *dc)
{
	SLLNode *current;
	current = begin_ptr;

	while (current != NULL)
	{
		current->mParticle->Draw(dc);
		current = current->mNext;
	}
}

/*
 * Name :        CParticlePool::Size()
 * Description : Returns the size of the list
 * Parameters :  none
 * Returns :     none
 */
int CParticlePool::Size()
{
	SLLNode *current;
	current = begin_ptr;
	int index = 0;

	while (current != NULL)
	{
		index++;
		current = current->mNext;
	}

	return index;
}

/*
 * Name :        CParticlePool::Clear()
 * Description : Dumps contents of this into target
 * Parameters :  target - Pool to dump to
 * Returns :     none
 */
void CParticlePool::Clear(CParticlePool* target)
{
	SLLNode *temp;
	temp = begin_ptr;

	while (temp != NULL)
	{
		target->AppendParticle(temp->mParticle);
		temp = temp->mNext;
	}
	begin_ptr = NULL;
}

/*
 * Name :        CParticlePool::Empty()
 * Description : Dumps contents of this
 * Parameters :  none
 * Returns :     none
 */
void CParticlePool::Empty()
{
	SLLNode *temp;
	temp = begin_ptr;

	while (temp != NULL)
	{
		SLLNode *hold = temp;
		temp = temp->mNext;
		delete hold;
	}
	begin_ptr = NULL;
}
