/*
 * Name:     CViewPlayer.h
 *
 *
 * Class:    CViewPlayer
 * Impl:     CViewPlayer.cpp
 * Created : Mar 21, 2010
 * Author :  Kenji Aono
 */

#pragma once
#ifndef CVIEWPLAYER_H_
#define CVIEWPLAYER_H_

#include "../CDocument.h"
#include "../CDocumentObserver.h"
#include "../Snow/CParticlePool.h"
#include "CViewActors.h"
#include "CViewEdit.h"
#include "CViewTimeline.h"

/*
 * Class:    CViewPlayer
 * 
 */
class CViewPlayer : public wxFrame, public CDocumentObserver
{
public:
	enum{ID_Timer, ID_Play, ID_Pause, ID_Stop, ID_Close};

	CViewPlayer(CDocument *document, wxWindow *parent, wxWindowID id,
			    const wxPoint & pos, const wxSize &size, CViewEdit *vEdit,
			    CViewTimeline *vTime, CViewActors *vAct, double tempTime);
	virtual ~CViewPlayer();

	void OnPaint(wxPaintEvent& event);
	void OnTimer(wxTimerEvent& event);
	void OnPlay(wxCommandEvent& event);
	void OnPause(wxCommandEvent& event);
	void OnStop(wxCommandEvent& event);
	void OnClose(wxCommandEvent& event) {Closer();}
	void OnExit(wxCloseEvent& event) {Closer();}
	void Closer();

	void CreateMoreSnow();
	void InitializePool();

	virtual void UpdateObserver();

	DECLARE_EVENT_TABLE()

private:
	CDocument *mDocument;
	CViewActors *mViewActors;
	CViewEdit *mViewEdit;
	CViewTimeline *mViewTimeline;
	wxWindow *mParent;

	wxTimer mTimer;
	wxStopWatch mTrueTime;
	wxButton* mPlay;
	wxButton* mPause;
	wxButton* mStop;
	wxButton* mClose;

	bool mPlaying;
	double mTempTime;
	CParticlePool mUsed;
	CParticlePool mUnUsed;
};

#endif /* CVIEWPLAYER_H_ */
